from requests import Response, HTTPError, Request


class InvalidInputException(Exception):
    pass


class UnauthenticatedException(Exception):
    pass


class SignatureErrorException(Exception):
    pass


class ResponseGeneralException(Exception):
    _raw_request: Request
    _raw_response: Response

    def __init__(self, exc: HTTPError):
        if isinstance(exc, HTTPError):
            self._raw_response = exc.response
            self._raw_request = exc.request

        super().__init__(exc)

    def get_raw_response(self) -> Response:
        return self._raw_response


class GeneralException(Exception):
    def __init__(
        self,
        original_exception,
        message="An error occurred",
    ):
        self.message = f"{message}: {original_exception}"
        super().__init__(self.message)



from py_sat.models.ping import PingResponse
from py_sat.models.inquiry import InquiryResponse, InquiryRequest
from py_sat.models.base import Field
from py_sat.models.order import OrderRequest, OrderDetail
from py_sat.models.error import ErrorResponse, ErrorObject
from py_sat.models.product import PartnerProduct, ProductListResponse, ProductStatus
from py_sat.models.account import Account



from setuptools import setup, find_packages

# read the contents of your README file
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="py_sat_sdk",
    packages=find_packages(),
    version="1.0.5",
    description="Python SDK for Tokopedia SAT API",
    author="SAT Team",
    install_requires=[
        "requests>=2.27.1",
        "requests-oauthlib>=2.0.0",
        "pycryptodome>=3.11.0",
        "python-dateutil>=2.9.0.post0",
    ],
    tests_require=[
        "pytest>=4.4.1",
        "pytest-dotenv>=0.5.2",
        "pytest-httpserver>=1.0.5",
    ],
    test_suite="tests",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.6",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
)



from typing import Union

from Crypto.Signature import pss
from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
import base64

from py_sat.signature.interface import SignatureAlgorithm


class PSSPaddingAlgorithm(SignatureAlgorithm):
    def verify(
        self, public_key: RSA, msg: Union[str, bytes], signature_base64: str
    ) -> bool:
        """Verifies a PKCS1v15 signature using SHA-256.

        Args:
            public_key: The RSA public key used for verification.
            msg: The message (either a string or bytes) to verify.
            signature_base64: The base64-encoded signature to verify.

        Returns:
            True if the signature is valid, False otherwise.
        """

        if not signature_base64.strip():
            raise ValueError("Signature is empty")

        if isinstance(msg, str):
            msg = msg.encode("utf-8")

        try:
            signature_bytes = base64.b64decode(signature_base64)
        except ValueError:
            raise ValueError("Invalid base64 signature")

        hash_message = SHA256.new(msg)
        salt = int(public_key.n.bit_length() / 8) - hash_message.digest_size - 2
        verifier = pss.new(public_key, salt_bytes=salt)

        try:
            verifier.verify(hash_message, signature_bytes)
            return True
        except (ValueError, TypeError):
            return False

    def sign(self, private_key: RSA, msg: Union[str, bytes]) -> str:
        """Signs a message using PSS padding and RSA algorithm.

        Args:
            private_key: The RSA private key used for signing.
            msg: The message (either a string or bytes) to sign.

        Returns:
            The base64-encoded signature as a string.
        """
        if isinstance(msg, str):
            msg = msg.encode("utf-8")

        hash = SHA256.new(msg)
        salt = int(private_key.n.bit_length() / 8) - hash.digest_size - 2
        signature = pss.new(private_key, salt_bytes=salt).sign(hash)

        base64_signature = base64.b64encode(signature)
        encoded_signature = str(base64_signature, "utf-8")
        return encoded_signature
